package com.welnz.device.example

import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.IntrinsicSize
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.verticalScroll
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Close
import androidx.compose.material.icons.filled.PlayArrow
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Surface
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import com.welnz.device.library.enums.ConnectionStatus
import com.welnz.device.example.extensions.isWhiteSpace
import com.welnz.device.example.ui.models.IDeviceViewModel
import com.welnz.device.example.ui.models.MockDeviceViewModel
import com.welnz.device.example.ui.theme.ExampleTheme
import java.math.BigDecimal
import java.math.RoundingMode

@Preview( showBackground = true )
@Composable
fun DeviceScreen(
    vm: IDeviceViewModel = MockDeviceViewModel()
){
    ExampleTheme {
        // A surface container using the 'background' color from the theme
        Surface(
            Modifier
                .fillMaxSize()
                .padding(all = 10.dp),
            color = MaterialTheme.colorScheme.background
        ) {
            Column(
                Modifier
                    .width(IntrinsicSize.Max),
                horizontalAlignment = Alignment.CenterHorizontally
            ) {
                DeviceHeaderDetails(vm)
                Spacer(
                    Modifier.height(25.dp)
                )
                Column(
                    Modifier
                        .verticalScroll(rememberScrollState()),
                    verticalArrangement = Arrangement.spacedBy(10.dp)
                ) {
                    if (vm.isGettingReady && !vm.isReady) {
                        Text(text = "Connecting to device...", fontWeight = FontWeight.Bold)
                    }
                    GeneralDeviceDetails(vm)
                    BatteryDetails(vm)
                    DeviceStatsDetails(vm)
                    MeasurementDetails(vm)
                    DeviceFooterDetails(vm)
                }
            }
        }
    }
}

@Preview( showBackground = true )
@Composable
fun DeviceHeaderDetails(
    vm: IDeviceViewModel = MockDeviceViewModel()
) {
    Text( fontWeight = FontWeight.Bold, text = vm.name )
    Text( text = vm.description )
    Text( text = "${vm.connectionStatus}" )
    Text( text = "${vm.status}" )
    Text( text = vm.firmware )
    Spacer(modifier = Modifier.height(24.dp))
}

@Preview( showBackground = true )
@Composable
fun DeviceFooterDetails(
    vm: IDeviceViewModel = MockDeviceViewModel()
) {
    if (vm.connectionStatus == ConnectionStatus.CONNECTED
        || vm.connectionStatus == ConnectionStatus.CONNECTING) {
        QuickLargeIconButton(
            icon = Icons.Filled.Close,
            text = "Disconnect",
            onClick = { vm.disconnect() }
        )
    } else {
        QuickLargeIconButton(
            icon = Icons.Filled.PlayArrow,
            text = "Connect",
            enabled = vm.connectionStatus == ConnectionStatus.DISCONNECTED,
            onClick = { vm.connect() }
        )
    }
    QuickLargeIconButton(
        icon = Icons.Filled.Close,
        text = "Close",
        onClick = { vm.close() }
    )
}

@Preview( showBackground = true )
@Composable
fun GeneralDeviceDetails (
    vm: IDeviceViewModel = MockDeviceViewModel()
){
    // Device Stats Details
    Row {
        Text( text = "Device Details", fontWeight = FontWeight.Bold )
    }
    deviceDetailRow("Serial Number:", "${vm.serialNumber?.serialNumber}")
    deviceDetailRow(
        "Calibration Date:",
        "${vm.calibrationDate?.date?.format(DATE_TIME_FORMATTER)}"
    )
    deviceDetailRow(
        "Check Date:",
        "${vm.checkDate?.date?.format(DATE_TIME_FORMATTER)}"
    )
    QuickIconButton(
        text = "Set Check Date",
        modifier = Modifier.fillMaxWidth(),
        onClick = { vm.setCheckDate() },
        enabled = vm.isReady
    )
    Spacer(modifier = Modifier.height( 15.dp ))
    QuickIconButton(
        text = "Fetch Device Details",
        modifier = Modifier.fillMaxWidth(),
        onClick = { vm.readGeneralDeviceDetails() },
        enabled = vm.isReady
    )
}

@Preview( showBackground = true )
@Composable
fun BatteryDetails(
    vm: IDeviceViewModel = MockDeviceViewModel()
) {
    // Battery Details
    Row {
        Text("Battery Status", fontWeight = FontWeight.Bold)
    }
    deviceDetailRow("Status:", "${vm.battery?.batteryState}")
    deviceDetailRow(
        "Percentage:",
        vm.battery?.let { "${it.batteryPercentage}%" } ?: ""
    )
    QuickIconButton(
        text = "Fetch Battery Status",
        modifier = Modifier.fillMaxWidth(),
        onClick = { vm.device.readBatteryPercentage() },
        enabled = vm.isReady
    )
}

@Preview( showBackground = true )
@Composable
fun DeviceStatsDetails (
    vm: IDeviceViewModel = MockDeviceViewModel()
){
    // Device Stats Details
    Row {
        Text( text = "Device Stats", fontWeight = FontWeight.Bold )
    }
    deviceDetailRow("Total Measurements:", "${vm.deviceStats?.totalMeasurements}")
    deviceDetailRow("Power Cycles:", "${vm.deviceStats?.powerCycles}")
    deviceDetailRow("USB Plug Cycles:", "${vm.deviceStats?.usbPlugCycles}")
    QuickIconButton(
        text = "Fetch Device Stats",
        modifier = Modifier.fillMaxWidth(),
        onClick = { vm.device.readStats() },
        enabled = vm.isReady
    )
}

@Preview( showBackground = true )
@Composable
fun MeasurementDetails (
    vm: IDeviceViewModel = MockDeviceViewModel()
){
    // Device Stats Details
    Row {
        Text( text = "Measurements", fontWeight = FontWeight.Bold )
    }

    // Only show error message when we have an error
    vm.measurement?.errorMessage?.let {
        if (!it.isWhiteSpace()) {
            deviceDetailRow("Error:", it)
        }
    }


    vm.measurement.let {
        deviceDetailRow( "kgf:", "${it?.kgForce?.let { it1 -> BigDecimal(it1).setScale(2, RoundingMode.HALF_EVEN) }}" )
        deviceDetailRow("GForce:", "${it?.gForce}" )
        deviceDetailRow("Floor:", "${it?.floor}" )

        deviceDetailRow("Width:", "${it?.width}")
        deviceDetailRow("Raw GForce:", "${it?.rawGForce}")
    }
    Row {
        QuickIconButton(
            text = "Fetch Last Measurement",
            modifier = Modifier.weight(0.5f),
            onClick = { vm.device.readLastMeasurement() },
            enabled = vm.isReady
        )
        Spacer( modifier = Modifier.width( 5.dp ) )
        QuickIconButton(
            text = "Take Measurement",
            modifier = Modifier.weight(0.5f),
            onClick = { vm.device.takeMeasurement() },
            enabled = vm.isReady
        )
    }
}