package com.welnz.device.example

import androidx.compose.foundation.BorderStroke
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.PlayArrow
import androidx.compose.material3.AssistChip
import androidx.compose.material3.AssistChipDefaults
import androidx.compose.material3.Button
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.shadow
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.vector.ImageVector
import androidx.compose.ui.text.font.FontStyle
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import com.welnz.device.example.extensions.isWhiteSpace
import com.welnz.device.example.ui.models.IDeviceViewModel
import com.welnz.device.example.ui.models.MockDeviceViewModel
import com.welnz.device.library.enums.ConnectionStatus

@Composable
fun deviceDetailRow(name: String, value: String, modifier: Modifier = Modifier) {
    Row ( modifier ) {
        Text( text = name, modifier = Modifier.weight(FIRST_COLUMN_WIDTH) )
        Text(
            text = if (!value.isWhiteSpace()) value else "-",
            modifier = Modifier.weight(SECOND_COLUMN_WIDTH)
        )
    }
}

@Preview(showBackground = true)
@Composable
fun DeviceRow(
    device: IDeviceViewModel = MockDeviceViewModel(),
    onConnectClicked: () -> Unit = {}
) {
    Row(
        modifier = Modifier.padding(all = 8.dp),
        verticalAlignment = Alignment.CenterVertically
    ) {
        Column {
            Row(
                verticalAlignment = Alignment.CenterVertically
            ) {
                Text(
                    text = device.name,
                    fontWeight = FontWeight.Bold
                )
            }
            Text(
                text = device.description,
                fontStyle = FontStyle.Italic
            )
        }
        Spacer(modifier = Modifier.weight(1.0f))
        QuickIconButton(
            text = "Connect",
            icon = Icons.Filled.PlayArrow,
            onClick = onConnectClicked,
            enabled = device.connectionStatus == ConnectionStatus.DISCONNECTED
        )
    }
}

@OptIn(ExperimentalMaterial3Api::class)
@Preview(showBackground = true)
@Composable
fun QuickIconChip(
    icon: ImageVector = Icons.Filled.PlayArrow,
    name: String = "Hello World",
    onClick: () -> Unit = {},
    modifier: Modifier = Modifier,
) {
    AssistChip(
        onClick,
        label = { Text(name) },
        modifier = modifier,
        leadingIcon = {
            Icon(
                icon,
                contentDescription = name,
                Modifier.size(AssistChipDefaults.IconSize)
            )
        }
    )
}

@Composable
fun QuickLargeIconButton(
    icon: ImageVector? = null,
    text: String = "Hello World",
    enabled: Boolean = true,
    onClick: () -> Unit = {},
    modifier: Modifier = Modifier,
    iconModifier: Modifier = Modifier,
    backgroundColor: Color = Color.White,

    tintColor: Color = Color.DarkGray,

    borderColor: Color = Color.DarkGray,
    fontColor: Color = Color.DarkGray,
) {
    QuickIconButton(
        icon,
        text,
        enabled,
        onClick,
        modifier.fillMaxWidth(),
        contentPadding = PaddingValues(top = 25.dp, bottom = 25.dp),
        iconModifier,

        backgroundColor,

        tintColor,
        borderColor,
        fontColor,
    )
}

@Composable
fun QuickIconButton(
    icon: ImageVector? = null,
    text: String = "Hello World",
    enabled: Boolean = true,
    onClick: () -> Unit = {},
    modifier: Modifier = Modifier,
    contentPadding: PaddingValues = PaddingValues(7.dp),
    iconModifier: Modifier = Modifier,

    backgroundColor: Color = Color.White,

    tintColor: Color = Color.DarkGray,
    borderColor: Color = Color.DarkGray,
    fontColor: Color = Color.DarkGray,
) {
    Button(
        onClick = onClick,
        enabled = enabled,
        modifier = modifier.shadow(0.dp),
        shape = RoundedCornerShape(12.dp),
        contentPadding = contentPadding,
        colors = ButtonDefaults.buttonColors(
            containerColor = backgroundColor,
            contentColor = fontColor
        ),
        border = BorderStroke(1.dp, borderColor)
    ) {
        Box(contentAlignment = Alignment.Center) {
            Row(modifier = Modifier.align(Alignment.CenterStart)) {
                Spacer(modifier = Modifier.width(4.dp))
                if (icon != null) {
                    Icon(
                        imageVector = icon,
                        modifier = iconModifier.size(18.dp),
                        contentDescription = "drawable_icons",
                        tint = tintColor,
                    )
                    Spacer(modifier = Modifier.width(4.dp))
                }
                Text( text, color = fontColor )
            }
        }
    }
}