package com.welnz.device.example.ui.models

import android.annotation.SuppressLint
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.setValue
import com.welnz.device.library.bluetooth.interfaces.SftDevice
import com.welnz.device.library.bluetooth.interfaces.SftDeviceDelegate
import com.welnz.device.library.enums.ConnectionStatus
import com.welnz.device.library.enums.LogType
import com.welnz.device.library.enums.Status
import com.welnz.device.library.response.interfaces.BatteryResponse
import com.welnz.device.library.response.interfaces.CalibrationDateResponse
import com.welnz.device.library.response.interfaces.CheckDateResponse
import com.welnz.device.library.response.interfaces.DeviceStatsResponse
import com.welnz.device.library.response.interfaces.FirmwareResponse
import com.welnz.device.library.response.interfaces.MeasurementResponse
import com.welnz.device.library.response.interfaces.SerialNumberResponse

import java.time.Instant
import java.time.LocalDate

@SuppressLint("MissingPermission")
class DeviceViewModel(
    private val parent: HomeViewModel,
    override val device: SftDevice
) : IDeviceViewModel,
    SftDeviceDelegate {
    override var name by mutableStateOf( device.name )
    override var description by mutableStateOf( device.description )

    override var lastMessage by mutableStateOf("")

    override var connectionStatus by mutableStateOf(ConnectionStatus.DISCONNECTED)
    override var isGettingReady by mutableStateOf(false)
    override var status by mutableStateOf(Status.NOT_READY)

    /***
     * Indicates the device is connected and ready for communication
     */
    override var isReady by mutableStateOf(false)

    override var firmware by mutableStateOf("-")
    override var battery by mutableStateOf<BatteryResponse?>( null )
    override var deviceStats by mutableStateOf<DeviceStatsResponse?>( null )
    override var measurement by mutableStateOf<MeasurementResponse?>( null )
    override var serialNumber by mutableStateOf<SerialNumberResponse?>( null )
    override var calibrationDate by mutableStateOf<CalibrationDateResponse?>( null )
    override var checkDate by mutableStateOf<CheckDateResponse?>( null )

    init {
        device.setDelegate(this)
    }

    private fun setIsReady() {
        // The device is ready only once it is fully connected
        isReady =
            connectionStatus == ConnectionStatus.CONNECTED
                    && status == Status.READY
    }

    override fun onConnectionStatus(connectionStatus: ConnectionStatus?) {
        if (connectionStatus != null) {
            isGettingReady = connectionStatus == ConnectionStatus.CONNECTING
                    || connectionStatus == ConnectionStatus.CONNECTED

            this.connectionStatus = connectionStatus
            setIsReady()
        }
    }

    override fun onStatus(status: Status?) {
        if (status != null) {
            this.status = status
            setIsReady()
        }
    }

    override fun onLog(timestamp: Instant?, type: LogType?, message: String?) {
        lastMessage = message ?: ""
    }

    override fun disconnect() {
        device.disconnect()
    }

    override fun close() {
        disconnect()
        parent.removeDevice(this)
    }

    override fun connect() {
        device.connect()
    }

    override fun setCheckDate() {
        device.writeCheckDate(LocalDate.now())
    }

    override fun readGeneralDeviceDetails() {
        device.readSerialNumber()
        device.readCalibrationDate()
        device.readCheckDate()
    }

    override fun onBattery(response: BatteryResponse?) {
        battery = response
    }

    override fun onDeviceStats(response: DeviceStatsResponse?) {
        deviceStats = response
    }

    override fun onMeasurement(response: MeasurementResponse?) {
        measurement = response
    }

    override fun onSerialNumber(response: SerialNumberResponse?) {
        serialNumber = response
    }

    override fun onCheckDate(response: CheckDateResponse?) {
        checkDate = response
    }

    override fun onCalibrationDate(response: CalibrationDateResponse?) {
        calibrationDate = response
    }

    override fun onFirmware(response: FirmwareResponse?) {
        firmware = response?.firmware ?: "-"
    }
}